---
subject: Follow the link to reset your password
---
{% use_layout %}

<{ mail_block center=True }>
    You have requested to reset your password. Please follow this link to proceed:
</{}>
<{ mail_action_block label="Reset your password" url=url_for("users.reset_password", token=token, _external=True) }/>
